package com.caimei365.user.service.impl;

import com.caimei365.user.components.RedisService;
import com.caimei365.user.components.WeChatService;
import com.caimei365.user.mapper.SellerMapper;
import com.caimei365.user.mapper.SuperVipMapper;
import com.caimei365.user.model.ResponseJson;
import com.caimei365.user.model.dto.ClubTemporaryDto;
import com.caimei365.user.model.po.ServiceProviderPo;
import com.caimei365.user.model.po.SuperVipPo;
import com.caimei365.user.model.vo.ClubTemporaryVo;
import com.caimei365.user.model.vo.ClubVo;
import com.caimei365.user.model.vo.ServiceProviderVo;
import com.caimei365.user.model.vo.UserLoginVo;
import com.caimei365.user.service.SellerService;
import com.caimei365.user.utils.JwtUtil;
import com.caimei365.user.utils.Md5Util;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/3/24
 */
@Slf4j
@Service
public class SellerServiceImpl implements SellerService {

    @Resource
    private RedisService redisService;
    @Resource
    private WeChatService weChatService;
    @Resource
    private SellerMapper sellerMapper;
    @Resource
    private SuperVipMapper vipMapper;

    /**
     * 协销机构列表
     *
     * @param serviceProviderId 协销Id
     * @param status            机构状态:
     *                          1:待审查资料, 2:电话预约, 3:已预约
     *                          20:待确认, 21:待拜访, 30:待员工推荐
     *                          90:已上线, 91:已冻结, 92:审查资料未通过,待补充资料, 40:已完成第一次采购
     *                          66:协销下所有上线的机构----虚拟状态
     * @param name              机构名字关键字（搜索用）
     * @param type              1.我的机构2.组员机构(组长展示全组，总管理员展示所有人)3.待分配机构
     * @return
     */
    @Override
    public ResponseJson<PageInfo<ClubVo>> getClubList(Integer serviceProviderId, Integer status, Integer type, String name, Integer userIdentity, Integer groupServiceId, Integer pageNum, Integer pageSize) {
        // 获取协销用户下的机构列表
        List<ClubVo> clubList = new ArrayList<ClubVo>();
        List<Integer> serviceIds = new ArrayList<Integer>();
        if (1 == type || 3 == type) {
            if(null != groupServiceId && groupServiceId > 0){
                return ResponseJson.success();
            }else{
                PageHelper.startPage(pageNum, pageSize);
                clubList = sellerMapper.findClubs(serviceProviderId, status, name, userIdentity, type, null);
            }
        } else if (2 == type) {
            if (null != groupServiceId && groupServiceId > 0) {
                PageHelper.startPage(pageNum, pageSize);
                clubList = sellerMapper.findClubs(groupServiceId, status, name, userIdentity, 1, null);
            } else {
                Integer leaderId = sellerMapper.findLeaderIdByServiceId(serviceProviderId);
                Integer managerId = sellerMapper.findmanagerIdByServiceId(serviceProviderId);
                if (null != leaderId) {
                    serviceIds = sellerMapper.findServiceIds(serviceProviderId);
                }
                if (null != managerId) {
                    serviceIds = sellerMapper.findAllServiceIds(serviceProviderId);
                }
                PageHelper.startPage(pageNum, pageSize);
                clubList = sellerMapper.findClubs(serviceProviderId, status, name, userIdentity, type, serviceIds);
            }
        }
        for (ClubVo club : clubList) {
            //未查看订单数量
            Integer orderNum = sellerMapper.findOrderNum(club);
            club.setOrderNum(orderNum);
            //查是不是vip
            SuperVipPo vip = vipMapper.findEndTime(club.getUserId());
            if (vip != null) {
                club.setVipFlag(1);
            } else {
                club.setVipFlag(0);
            }
            String newClubId = club.getClubId().toString();
            int maxLen = 6;
            int len = newClubId.length();
            if (len < maxLen) {
                for (int i = 0;i < (maxLen - len); i++) {
                    newClubId = "0" + newClubId;
                }
            }
            club.setNewClubId(newClubId);
        }
        PageInfo<ClubVo> pageData = new PageInfo<>(clubList);
        return ResponseJson.success(pageData);
    }

    /**
     * 协销登录(手机号，密码)
     *
     * @param mobile   手机号
     * @param password 密码
     * @param unionId  微信unionId
     * @return UserLoginVo
     */
    @Override
    public ResponseJson<UserLoginVo> passwordLogin(String mobile, String password, String unionId) {
        if (StringUtils.isBlank(mobile) || StringUtils.isBlank(password)) {
            return ResponseJson.error("请输入账号密码", null);
        }
        if (StringUtils.isBlank(unionId)) {
            return ResponseJson.error("请输入微信unionId", null);
        }
        UserLoginVo seller = sellerMapper.getLoginSellerByMobile(mobile);
        if (null == seller || !Md5Util.md5(password).equals(seller.getPassword())) {
            return ResponseJson.error("密码和账户名不匹配", null);
        }
        // 生成token
        String token = JwtUtil.createToken(seller.getUserId());
        // 为了过期续签，将token存入redis，并设置超时时间
        redisService.set(token, token, JwtUtil.getExpireTime());
        seller.setToken(token);
        Map<Object, Object> infoData = redisService.getEntries("wxInfo:applets:" + unionId);
        String openId = (String) infoData.get(WeChatService.Keys.OPEN_ID);
        sellerMapper.updateServiceProviderByUserId(seller.getUserId(), openId, unionId);
        log.info("协销账号密码登录openid>>>>" + openId + " ,unionId>>>>>" + unionId);
        seller.setManager(sellerMapper.findManager(mobile));
        return ResponseJson.success(seller);
    }

    /**
     * 待注册机构列表
     *
     * @param userId 协销用户id
     * @return ClubTemporaryVo
     */
    @Override
    public ResponseJson<PageInfo<ClubTemporaryVo>> temporaryClub(Integer userId, String searchName, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<ClubTemporaryVo> clubTemporaryList = sellerMapper.findClubTemporary(userId, searchName);
        PageInfo<ClubTemporaryVo> pageInfo = new PageInfo<>(clubTemporaryList);
        return ResponseJson.success(pageInfo);
    }

    @Override
    public ResponseJson<String> modifyTemporaryClub(ClubTemporaryDto temporaryDto) {
        sellerMapper.updateClubTemporary(temporaryDto);
        return ResponseJson.success("");
    }

    @Override
    public ResponseJson<String> deleteTemporaryClub(Integer id) {
        sellerMapper.deleteTemporaryClub(id);
        return ResponseJson.success("");
    }

    /**
     * 所有的机构列表
     *
     * @param searchWord 搜索关键字
     * @param pageNum
     * @param pageSize
     */
    @Override
    public ResponseJson<PageInfo<ClubVo>> getAllClubList(String searchWord, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<ClubVo> clubList = sellerMapper.getAllClubList(searchWord);
        PageInfo<ClubVo> pageData = new PageInfo<>(clubList);
        return ResponseJson.success(pageData);
    }

    /**
     * 协销个人中心
     *
     * @param userId 协销用户id
     */
    @Override
    public ResponseJson<ServiceProviderPo> getSellerHome(Integer userId) {
        // 协销用户
        ServiceProviderPo seller = sellerMapper.getSellerByUserId(userId);
        if (null == seller) {
            return ResponseJson.error("未查询到此协销人员！", null);
        }
        return ResponseJson.success(seller);
    }

    @Override
    public ResponseJson<List<ServiceProviderVo>> getServiceTeam(Integer spId,Integer status) {
        Integer leaderId = sellerMapper.findLeaderIdByServiceId(spId);
        Integer managerId = sellerMapper.findmanagerIdByServiceId(spId);
        List<ServiceProviderVo> serviceProviderVos = new ArrayList<>();
        if (null != leaderId) {
            serviceProviderVos = sellerMapper.findServices(spId,status);
        }
        if (null != managerId) {
            serviceProviderVos = sellerMapper.findAllServices(spId,status);
        }
        //个人返回空，组长返回小组所有人
        return ResponseJson.success(serviceProviderVos);
    }

}
